@@ -1,25 +1,26 @@
 /* gda-parameter.c
  *
- * Copyright (C) 2003 - 2006 Vivien Malerba
+ * Copyright (C) 2003 - 2007 Vivien Malerba
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
+ * This Library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This Library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * You should have received a copy of the GNU Library General Public
+ * License along with this Library; see the file COPYING.LIB.  If not,
+ * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include <string.h>
 #include "gda-parameter.h"
+#include "gda-parameter-util.h"
 #include "gda-dict.h"
 #include "gda-dict-type.h"
 #include "gda-query.h"
@@ -93,6 +94,7 @@ enum
 	PROP_RESTRICT_MODEL,
 	PROP_RESTRICT_COLUMN,
 	PROP_GDA_TYPE,
+	PROP_NOT_NULL,
 	PROP_DICT_TYPE /* TO ADD */
 };
 
@@ -208,22 +210,29 @@ gda_parameter_class_init (GdaParameterClass *class)
 	g_object_class_install_property (object_class, PROP_USE_DEFAULT_VALUE,
 					 g_param_spec_boolean ("use_default_value", NULL, NULL, FALSE,
 							       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
+	g_object_class_install_property (object_class, PROP_NOT_NULL,
+					 g_param_spec_boolean ("not_null", NULL, NULL, FALSE,
+							       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	g_object_class_install_property (object_class, PROP_SIMPLE_BIND,
-					 g_param_spec_pointer ("simple_bind", NULL, NULL, 
+					 g_param_spec_object ("simple_bind", NULL, NULL, 
+                                                               GDA_TYPE_PARAMETER,
 							       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	g_object_class_install_property (object_class, PROP_FULL_BIND,
-					 g_param_spec_pointer ("full_bind", NULL, NULL, 
+					 g_param_spec_object ("full_bind", NULL, NULL, 
+                                                               GDA_TYPE_PARAMETER,
 							       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	g_object_class_install_property (object_class, PROP_RESTRICT_MODEL,
-                                         g_param_spec_pointer ("restrict_model", NULL, NULL,
+                                         g_param_spec_object ("restrict_model", NULL, NULL,
+                                                               GDA_TYPE_DATA_MODEL,
                                                                (G_PARAM_READABLE | G_PARAM_WRITABLE)));
         g_object_class_install_property (object_class, PROP_RESTRICT_COLUMN,
                                          g_param_spec_int ("restrict_column", NULL, NULL,
 							   0, G_MAXINT, 0,
 							   (G_PARAM_READABLE | G_PARAM_WRITABLE)));
-	/* class attributes */
+
 	/* class attributes */
 	GDA_OBJECT_CLASS (class)->id_unique_enforced = FALSE;
+	GDA_OBJECT_CLASS (class)->name_as_initial_id = TRUE;
 }
 
 static void
@@ -480,6 +489,9 @@ gda_parameter_set_property    (GObject *object,
 				parameter->priv->plugin = g_strdup (ptr);
 			break;
 		case PROP_USE_DEFAULT_VALUE:
+			if (g_value_get_boolean (value) == parameter->priv->default_forced)
+				break;
+
 			if (g_value_get_boolean (value)) {
 				if (!parameter->priv->has_default_value)
 					g_warning ("Can't force parameter to use default value if there "
@@ -491,19 +503,38 @@ gda_parameter_set_property    (GObject *object,
 			}
 			else
 				parameter->priv->default_forced = FALSE;
-
+			gda_object_signal_emit_changed (GDA_OBJECT (parameter));
 			break;
+		case PROP_NOT_NULL: {
+			gboolean not_null = g_value_get_boolean (value);
+			if (not_null != parameter->priv->not_null) {
+				parameter->priv->not_null = not_null;
+				
+				/* updating the parameter's validity regarding the NULL value */
+				if (!not_null && 
+				    (!parameter->priv->value || gda_value_is_null (parameter->priv->value)))
+					parameter->priv->valid = TRUE;
+				
+				if (not_null && 
+				    (!parameter->priv->value || gda_value_is_null (parameter->priv->value)))
+					parameter->priv->valid = FALSE;
+				
+				gda_object_signal_emit_changed (GDA_OBJECT (parameter));
+			}
+			break;
+		}
 		case PROP_SIMPLE_BIND:
-			gda_parameter_bind_to_param (parameter, g_value_get_pointer (value));
+			gda_parameter_bind_to_param (parameter, GDA_PARAMETER (g_value_get_object (value)));
 			break;
 		case PROP_FULL_BIND:
-			gda_parameter_set_full_bind_param (parameter, g_value_get_pointer (value));
+			gda_parameter_set_full_bind_param (parameter, GDA_PARAMETER (g_value_get_object (value)));
 			break;
-		case PROP_RESTRICT_MODEL:
-			ptr = g_value_get_pointer (value);
+		case PROP_RESTRICT_MODEL: {
+			GdaDataModel* ptr = g_value_get_object (value);
 			g_return_if_fail (gda_parameter_restrict_values (parameter, 
 									 (GdaDataModel *)ptr, -1, NULL));
 			break;
+                }
 		case PROP_RESTRICT_COLUMN:
 			parameter->priv->restrict_col = g_value_get_int (value);
 			break;
@@ -530,14 +561,17 @@ gda_parameter_get_property    (GObject *object,
 		case PROP_USE_DEFAULT_VALUE:
 			g_value_set_boolean (value, parameter->priv->default_forced);
 			break;
+		case PROP_NOT_NULL:
+			g_value_set_boolean (value, gda_parameter_get_not_null (parameter));
+			break;
 		case PROP_SIMPLE_BIND:
-			g_value_set_pointer (value, parameter->priv->change_with);
+			g_value_set_object (value, G_OBJECT (parameter->priv->change_with));
 			break;
 		case PROP_FULL_BIND:
-			g_value_set_pointer (value, parameter->priv->alias_of);
+			g_value_set_object (value, G_OBJECT (parameter->priv->alias_of));
 			break;
 		case PROP_RESTRICT_MODEL:
-			g_value_set_pointer (value, parameter->priv->restrict_model);
+			g_value_set_object (value, G_OBJECT (parameter->priv->restrict_model));
 			break;
 		case PROP_RESTRICT_COLUMN:
 			g_value_set_int (value, parameter->priv->restrict_col);
@@ -650,6 +684,39 @@ gda_parameter_get_value (GdaParameter *param)
 		return gda_parameter_get_value (param->priv->alias_of);
 }
 
+/**
+ * gda_parameter_get_value_str
+ * @param: a #GdaParameter object
+ * 
+ * Get a string representation of the value stored in @param. Calling
+ * gda_parameter_set_value_str () with this value will restore @param's current
+ * state.
+ *
+ * Returns: a new string, or %NULL if @param's value is NULL
+ */
+gchar *
+gda_parameter_get_value_str (GdaParameter *param)
+{
+	const GValue *current_val;
+
+	g_return_val_if_fail (GDA_IS_PARAMETER (param), NULL);
+	g_return_val_if_fail (param->priv, NULL);
+
+	current_val = gda_parameter_get_value (param);
+	if (!current_val)
+		return NULL;
+	else {
+		GdaDict *dict;
+		GdaDataHandler *dh;
+
+		dict = gda_object_get_dict (GDA_OBJECT (param));
+		dh = gda_dict_get_handler (dict, param->priv->g_type);
+		if (dh)
+			return gda_data_handler_get_str_from_value (dh, current_val);
+		else
+			return NULL;
+	}
+}
 
 /*
  * gda_parameter_set_value
@@ -703,8 +770,9 @@ gda_parameter_set_value (GdaParameter *param, const GValue *value)
 
 	if (value &&
 	    (G_VALUE_TYPE ((GValue *)value) != GDA_TYPE_NULL) &&
-	    (G_VALUE_TYPE ((GValue *)value) != param->priv->g_type))
+	    (G_VALUE_TYPE ((GValue *)value) != param->priv->g_type)) {
 		param->priv->valid = FALSE;
+	}
 
 #ifdef DEBUG_PARAM
 	g_print ("Changed param %p (%s): value %s --> %s \t(type %d -> %d) VALID: %d->%d CHANGED: %d\n", 
@@ -778,7 +846,7 @@ gda_parameter_set_value_str (GdaParameter *param, const gchar *value)
 	g_return_val_if_fail (GDA_IS_PARAMETER (param), FALSE);
 	g_return_val_if_fail (param->priv, FALSE);
 
-	if (!value) {
+	if (!value || !g_ascii_strcasecmp (value, "NULL")) {
 		gda_parameter_set_value (param, NULL);	
 		return TRUE;
 	}
@@ -861,7 +929,7 @@ gda_parameter_is_valid (GdaParameter *param)
 		}
 
 		if (param->priv->default_forced) {
-			return param->priv->default_value ? TRUE : FALSE;
+			return (param->priv->default_value || param->priv->has_default_value) ? TRUE : FALSE;
 		}
 		else {
 			return param->priv->valid;
@@ -969,20 +1037,7 @@ gda_parameter_set_not_null (GdaParameter *param, gboolean not_null)
 	g_return_if_fail (GDA_IS_PARAMETER (param));
 	g_return_if_fail (param->priv);
 
-	if (not_null != param->priv->not_null) {
-		param->priv->not_null = not_null;
-
-		/* updating the parameter's validity regarding the NULL value */
-		if (!not_null && 
-		    (!param->priv->value || gda_value_is_null (param->priv->value)))
-			param->priv->valid = TRUE;
-
-		if (not_null && 
-		    (!param->priv->value || gda_value_is_null (param->priv->value)))
-			param->priv->valid = FALSE;
-
-		gda_object_signal_emit_changed (GDA_OBJECT (param));
-	}
+	g_object_set (G_OBJECT (param), "not_null", not_null, NULL);
 }
 
 /**
@@ -1313,9 +1368,10 @@ gda_parameter_dump (GdaParameter *parameter, guint offset)
 		gchar *strval;
 
 		strval = gda_value_stringify ((GValue *) gda_parameter_get_value (parameter));
-		g_print ("%s" D_COL_H1 "GdaParameter %p (%s), type=%s, %s, value=%s\n" D_COL_NOR, str, parameter,
+		g_print ("%s" D_COL_H1 "GdaParameter %p (%s/%s), type=%s, %s, value=%s\n" D_COL_NOR, str, parameter,
+			 gda_object_get_id (GDA_OBJECT (parameter)), 
 			 gda_object_get_name (GDA_OBJECT (parameter)), 
-			 g_type_to_string (parameter->priv->g_type),
+			 gda_g_type_to_string (parameter->priv->g_type),
 			 gda_parameter_is_valid (parameter) ? "VALID" : "INVALID",
 			 strval);
 		g_free (strval);
@@ -1363,6 +1419,34 @@ gda_parameter_dump (GdaParameter *parameter, guint offset)
 }
 #endif
 
+/**
+ * gda_parameter_get_alphanum_name
+ * @param: a #GdaParameter object
+ *
+ * Get a new string containing a "clean" version of @param's name: chars which
+ * are not among [0-9A-Za-z] are replaced with '_'.
+ *
+ * Returns: a new string
+ */
+gchar *
+gda_parameter_get_alphanum_name (GdaParameter *param)
+{
+	g_return_val_if_fail (GDA_IS_PARAMETER (param), NULL);
+
+	const gchar* ptr = gda_object_get_name (GDA_OBJECT (param));
+	gchar* ret = NULL;
+	if (ptr)
+		ret = g_strdup (ptr);
+	else
+		ret = NULL;
+
+	for (ptr = ret; ptr && *ptr; ptr++)
+		if (! (((*ptr >= '0') || (*ptr <= '9')) ||
+		       ((*ptr >= 'A') || (*ptr <= 'Z')) ||
+		       ((*ptr >= 'a') || (*ptr <= 'z')))) 
+			*ret = '_';
+	return ret;	
+}
 
 /* 
  * GdaReferer interface implementation
